package workflow_execution

import (
	"encoding/json"
	"strings"
	"time"

	"cloud.o-forge.io/core/oc-lib/models/utils"
	"cloud.o-forge.io/core/oc-lib/tools"
	"github.com/google/uuid"
)

type ScheduledType int

const (
	SCHEDULED ScheduledType = iota + 1
	STARTED
	FAILURE
	SUCCESS
)

var str = [...]string{
	"scheduled",
	"started",
	"failure",
	"success",
}

func FromInt(i int) string {
	return str[i]
}

func (d ScheduledType) String() string {
	return str[d]
}

// EnumIndex - Creating common behavior - give the type a EnumIndex functio
func (d ScheduledType) EnumIndex() int {
	return int(d)
}

type WorkflowExecutions struct {
	WorkflowID string               `json:"workflow_id" bson:"workflow_id"`
	ResourceID string               `json:"resource_id" bson:"resource_id"`
	Executions []*WorkflowExecution `json:"executions" bson:"executions"`
}

func (dma *WorkflowExecutions) Deserialize(j map[string]interface{}) *WorkflowExecutions {
	b, err := json.Marshal(j)
	if err != nil {
		return nil
	}
	json.Unmarshal(b, dma)
	return dma
}

func (dma *WorkflowExecutions) Serialize() map[string]interface{} {
	var m map[string]interface{}
	b, err := json.Marshal(dma)
	if err != nil {
		return nil
	}
	json.Unmarshal(b, &m)
	return m
}

type WorkflowExecution struct {
	utils.AbstractObject
	ExecDate   *time.Time `json:"execution_date,omitempty" bson:"execution_date,omitempty" validate:"required"`
	EndDate    *time.Time `json:"end_date,omitempty" bson:"end_date,omitempty"`
	State      int64      `json:"state,omitempty" bson:"state,omitempty"`
	WorkflowID string     `json:"workflow_id" bson:"workflow_id,omitempty"`
}

func (wfa *WorkflowExecution) ArgoStatusToState(status string) *WorkflowExecution {
	status = strings.ToLower(status)
	switch status {
	case "succeeded": // Succeeded
		wfa.State = int64(SUCCESS.EnumIndex())
	case "pending": // Pending
		wfa.State = int64(SCHEDULED.EnumIndex())
	case "running": // Running
		wfa.State = int64(STARTED.EnumIndex())
	default: // Failed
		wfa.State = int64(FAILURE.EnumIndex())
	}
	return wfa
}

func (ao *WorkflowExecution) GetID() string {
	return ao.UUID
}

func (r *WorkflowExecution) GenerateID() {
	r.UUID = uuid.New().String()
}

func (d *WorkflowExecution) GetName() string {
	return d.UUID + "_" + d.ExecDate.String()
}

func (d *WorkflowExecution) GetAccessor(caller *tools.HTTPCaller) utils.Accessor {
	data := New()
	data.Init(utils.WORKFLOW_EXECUTION, caller)
	return data
}

func (dma *WorkflowExecution) Deserialize(j map[string]interface{}) utils.DBObject {
	b, err := json.Marshal(j)
	if err != nil {
		return nil
	}
	json.Unmarshal(b, dma)
	return dma
}

func (dma *WorkflowExecution) Serialize() map[string]interface{} {
	var m map[string]interface{}
	b, err := json.Marshal(dma)
	if err != nil {
		return nil
	}
	json.Unmarshal(b, &m)
	return m
}
